<?php
/*
* 2007-2015 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2015 PrestaShop SA
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

class ipasspay_host extends PaymentModule
{
    protected $_html = '';
    protected $_postErrors = array();

    public $mid;
    public $app_id;
    public $api_secret;
    public $gateway_url;
    public $gateway_version;
    public $asyn_url;
    //public $syn_url;
    public $details;
    public $owner;
    public $address;
    public $extra_mail_vars;

    public function __construct()
    {
        $this->name = 'ipasspay_host';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.0';
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
        $this->author = 'PrestaShop';
        $this->controllers = array('validation');
        //$this->controllers = array('validation');
        $this->is_eu_compatible = 1;

        $this->currencies = true;
        $this->currencies_mode = 'checkbox';

        //$config = Configuration::getMultiple(array('IPASSPAY_HOST_MERCHANT_ID', 'IPASSPAY_HOST_APP_ID', 'IPASSPAY_HOST_API_SECRET', 'IPASSPAY_HOST_GATEWAY_URL', 'IPASSPAY_HOST_GATEWAY_VERSION','IPASSPAY_HOST_ASYN_URL','IPASSPAY_SYN_URL'));
        $config = Configuration::getMultiple(array('IPASSPAY_HOST_MERCHANT_ID', 'IPASSPAY_HOST_APP_ID', 'IPASSPAY_HOST_API_SECRET', 'IPASSPAY_HOST_GATEWAY_URL', 'IPASSPAY_HOST_GATEWAY_VERSION','IPASSPAY_HOST_ASYN_URL'));

        if (!empty($config['IPASSPAY_HOST_MERCHANT_ID'])) {
            $this->mid = $config['IPASSPAY_HOST_MERCHANT_ID'];
        }

        if (!empty($config['IPASSPAY_HOST_APP_ID'])) {
            $this->app_id = $config['IPASSPAY_HOST_APP_ID'];
        }

        if (!empty($config['IPASSPAY_HOST_API_SECRET'])) {
            $this->api_secret = $config['IPASSPAY_HOST_API_SECRET'];
        }

        if (!empty($config['IPASSPAY_HOST_GATEWAY_URL'])) {
            $this->gateway_url = $config['IPASSPAY_HOST_GATEWAY_URL'];
        }

        if (!empty($config['IPASSPAY_HOST_GATEWAY_VERSION'])) {
            $this->gateway_version = $config['IPASSPAY_HOST_GATEWAY_VERSION'];
        }

        if (!empty($config['IPASSPAY_HOST_ASYN_URL'])) {
            $this->asyn_url = $config['IPASSPAY_HOST_ASYN_URL'];
        }

        //if (!empty($config['IPASSPAY_SYN_URL'])) {
        //    $this->syn_url = $config['IPASSPAY_SYN_URL'];
       // }

        $this->bootstrap = true;
        parent::__construct();


        $this->displayName = $this->l('Credit Card Payment(iPasspay Host)');
        $this->description = $this->l('Accept payments for your products via credit card.');
        $this->confirmUninstall = $this->l('Are you sure about removing these details?');
        if (!isset($this->mid) || !isset($this->app_id) || !isset($this->api_secret) || !isset($this->gateway_url) || !isset($this->gateway_version)) {
            $this->warning = $this->l('The mandantory fields must be configured before using this module.');
        }

        if (!count(Currency::checkPaymentCurrencies($this->id))) {
            $this->warning = $this->l('No currency has been set for this module.');
        }
    }

    public function install()
    {
        if (!parent::install() || !$this->registerHook('paymentOptions') || !$this->registerHook('paymentReturn')) {
            return false;
        }
        return true;
    }

    protected function _postValidation() {
        if (Tools::isSubmit('btnSubmit')) {
            if (!Tools::getValue('IPASSPAY_HOST_MERCHANT_ID')) {
                $this->_postErrors[] = $this->l('Merchant ID is required.');
            } elseif (!Tools::getValue('IPASSPAY_HOST_APP_ID')) {
                $this->_postErrors[] = $this->l('Site id is required.');
            } elseif (!Tools::getValue('IPASSPAY_HOST_API_SECRET')) {
                $this->_postErrors[] = $this->l('API KEY is required.');
            } elseif (!Tools::getValue('IPASSPAY_HOST_GATEWAY_URL')) {
                $this->_postErrors[] = $this->l('Gateway URL is required.');
            } elseif (!Tools::getValue('IPASSPAY_HOST_GATEWAY_VERSION')) {
                $this->_postErrors[] = $this->l('Gateway version is required.');
            }elseif (!Tools::getValue('IPASSPAY_HOST_ASYN_URL')) {
                $this->_postErrors[] = $this->l('Asyn url is required.');
            }//elseif (!Tools::getValue('IPASSPAY_SYN_URL')) {
             //   $this->_postErrors[] = $this->l('Syn url is required.');
            //}


        }
    }

    protected function _postProcess() {
        if (Tools::isSubmit('btnSubmit')) {
            Configuration::updateValue('IPASSPAY_HOST_MERCHANT_ID', Tools::getValue('IPASSPAY_HOST_MERCHANT_ID'));
            Configuration::updateValue('IPASSPAY_HOST_APP_ID', Tools::getValue('IPASSPAY_HOST_APP_ID'));
            Configuration::updateValue('IPASSPAY_HOST_API_SECRET', Tools::getValue('IPASSPAY_HOST_API_SECRET'));
            Configuration::updateValue('IPASSPAY_HOST_GATEWAY_URL', Tools::getValue('IPASSPAY_HOST_GATEWAY_URL'));
            Configuration::updateValue('IPASSPAY_HOST_GATEWAY_VERSION', Tools::getValue('IPASSPAY_HOST_GATEWAY_VERSION'));
            Configuration::updateValue('IPASSPAY_HOST_ASYN_URL', Tools::getValue('IPASSPAY_HOST_ASYN_URL'));
            //Configuration::updateValue('IPASSPAY_SYN_URL', Tools::getValue('IPASSPAY_SYN_URL'));

        }
        $this->_html .= $this->displayConfirmation($this->l('Settings updated'));
    }

    protected function _displayIpasspay() {

        return $this->display(__FILE__, 'infos.tpl');
    }

    public function getContent() {

        if (Tools::isSubmit('btnSubmit')) {
            $this->_postValidation();
            if (!count($this->_postErrors)) {
                $this->_postProcess();
            } else {
                foreach ($this->_postErrors as $err) {
                    $this->_html .= $this->displayError($err);
                }
            }

        } else {
            $this->_html .= '<br />';
        }

        $this->_html .= $this->_displayIpasspay();
        $this->_html .= $this->renderForm();
        return $this->_html;
    }

    public function hookPaymentOptions($params)
    {

        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }
        $payment_options = [
            $this->getEmbeddedPaymentOption(),
        ];
        return $payment_options;
    }

    public function checkCurrency($cart)
    {

        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    public function getEmbeddedPaymentOption()
    {
        $embeddedOption = new PaymentOption();
        $embeddedOption->setCallToActionText($this->l('Credit Card (iPasspay Host)'))
                       ->setForm($this->generateForm())
                       ->setAdditionalInformation($this->context->smarty->fetch('module:ipasspay_host/views/templates/front/payment_infos.tpl'))
                       ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_.$this->name.'/payment.jpg'));

        return $embeddedOption;
    }

    //前台页面付款表单展示
    protected function generateForm()
    {
        /*
        $months = [];
        for ($i = 1; $i <= 12; $i++) {
            $months[] = sprintf("%02d", $i);
        }

        $years = [];
        for ($i = 0; $i <= 10; $i++) {
            $years[] = date('Y', strtotime('+'.$i.' years'));
        }
        */
        $this->context->smarty->assign([
            'action' => $this->context->link->getModuleLink($this->name, 'validation', array(), true),
            //'months' => $months,
            //'years' => $years,

        ]);

        return $this->context->smarty->fetch('module:icronpay/views/templates/front/payment_form.tpl');
    }

    public function renderForm() {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Contact details'),
                    'icon' => 'icon-envelope',
                ),
                'input' => array(
                    array(
                        'type' => 'text',
                        'label' => $this->l('Merchant ID'),
                        'name' => 'IPASSPAY_HOST_MERCHANT_ID',//IPASSPAY_MERCHANT_ID
                        'desc' => "Your merchant id which is used to sign in ipasspay_host",
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('APP ID'),
                        'name' => 'IPASSPAY_HOST_APP_ID', //IPASSPAY_SITE_ID
                        'desc' => "Auto generated when you add website in ipasspay_host",
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('API SECRET'),
                        'name' => 'IPASSPAY_HOST_API_SECRET', //IPASSPAY_API_KEY
                        'desc' => "You can find it in the dashboard of ipasspay_host",
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Gateway URL'),
                        'name' => 'IPASSPAY_HOST_GATEWAY_URL', //IPASSPAY_GATEWAY_URL
                        'desc' => "Set it as: https://service.ipasspay.biz/gateway/Index/checkout",
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Gateway Version'),
                        'name' => 'IPASSPAY_HOST_GATEWAY_VERSION', //IPASSPAY_GATEWAY_VERSION
                        'desc' => "Get your Gateway version from your ipasspay account. Normally, please set it to 1.0",
                        'required' => true,
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Notify URL'),
                        'name' => 'IPASSPAY_HOST_ASYN_URL', //IPASSPAY_ASYN_URL
                        'desc' => "We will carry transaction information and return to this interface when the transaction is completed.
                         Such as: https://yourdomain.com/index.php?fc=module&module=ipasspay_host&controller=validation&asyn_notify=1",
                        'required' => true,
                    ),

                    /*
                    array(
                        'type' => 'text',
                        'label' => $this->l('Syn url'),
                        'name' => 'IPASSPAY_HOST_SYN_URL', //IPASSPAY_SYN_URL
                        'desc' => "Syn Url",
                        'required' => true,
                    ),
                    */

                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->id = (int) Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($fields_form));
    }

    public function getConfigFieldsValues() {
        return array(
            'IPASSPAY_HOST_MERCHANT_ID' => Tools::getValue('IPASSPAY_HOST_MERCHANT_ID', Configuration::get('IPASSPAY_HOST_MERCHANT_ID')),
            'IPASSPAY_HOST_APP_ID' => Tools::getValue('IPASSPAY_HOST_APP_ID', Configuration::get('IPASSPAY_HOST_APP_ID')),
            'IPASSPAY_HOST_API_SECRET' => Tools::getValue('IPASSPAY_HOST_API_SECRET', Configuration::get('IPASSPAY_HOST_API_SECRET')),
            'IPASSPAY_HOST_GATEWAY_URL' => Tools::getValue('IPASSPAY_HOST_GATEWAY_URL', Configuration::get('IPASSPAY_HOST_GATEWAY_URL')),
            'IPASSPAY_HOST_GATEWAY_VERSION' => Tools::getValue('IPASSPAY_HOST_GATEWAY_VERSION', Configuration::get('IPASSPAY_HOST_GATEWAY_VERSION')),
            'IPASSPAY_HOST_ASYN_URL' => Tools::getValue('IPASSPAY_HOST_ASYN_URL', Configuration::get('IPASSPAY_HOST_ASYN_URL')),
            //'IPASSPAY_SYN_URL' => Tools::getValue('IPASSPAY_SYN_URL', Configuration::get('IPASSPAY_SYN_URL')),

        );
    }
}
